// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * Departure-train category compatibility.
 *
 * The categories of trains must be compatible with the departures they
 * are assigned to.
 *
 * @author YDS
 *
 */
public class DepartureTrainCategoryCompatibilityConstraint
        implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public DepartureTrainCategoryCompatibilityConstraint(
            final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals("Departure")) {
                    Departure dep = departureDAO.loadByName(
                            fill.getComplement());
                    if (dep != null) {
                        if (!dep.getCompCatDep().contains(
                                train.getCategory())) {
                            logger.log(Level.INFO, "Departure - train "
                                    + "category compatibility error : "
                                    + train.getName() + " category ("
                                    + train.getCategory().getName()
                                    + ") is not compatible with "
                                    + dep.getName() + ".");
                            valid = false;
                        }
                    } else {
                        logger.log(Level.INFO, "Departure - train category "
                        		+ "compatibility error : "
                                + fill.getTrain().getName()
                                + " departure event is associated with "
                                + fill.getComplement()
                                + " which is not a departure "
                                + "at " + fill.getDate().toString() + ".");
                        valid = false;
                    }
                }
            }
        }

        return valid;
    }

}
