// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.IndividualTrack;
import com.sncf.checker.bo.ImposedConsumption;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.ImposedConsumptionDAO;

/**
 * Imposed consumption.
 *
 * @author YDS
 *
 */
public class ImposedConsumptionConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public ImposedConsumptionConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        ImposedConsumptionDAO consumptionDAO =
                factory.getImposedConsumptionDAO();
        for (ImposedConsumption consumption : consumptionDAO.loadAll()) {
            if (consumption.getResource().getClass().getSuperclass()
                    == IndividualTrack.class) {
                List<Fill> fillList = consumption.getResource().getFill();
                Datetime prev = Datetime.MIN;
                int nbTrain = 0;
                Collections.sort(fillList, new SortFillByDate());
                for (Fill fill : fillList) {
                    if (nbTrain > 0) {
                        if (prev.getTime() < consumption.getEnd().getTime()
                                && fill.getDate().getTime()
                                > consumption.getBeg().getTime()) {
                            logger.log(Level.INFO, "Imposed consumption error : "
                                    + "no train should be using "
                                    + consumption.getResource().getName()
                                    + " between "
                                    + consumption.getBeg().toString()
                                    + " and "
                                    + consumption.getEnd().toString());
                                valid = false;
                                break;
                        }
                    }
                    if (fill.getEvent().equals("EnterResource")) {
                        nbTrain++;
                    } else if (fill.getEvent().equals("ExitResource")) {
                        nbTrain--;
                    }

                    prev = fill.getDate();
                }
            }
        }
        return valid;
    }

}
