// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.SingleTrack;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.SingleTrackDAO;

public class SingleTrackCapacityConstraint implements Constraint {

    private DAOFactory factory;

    public SingleTrackCapacityConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        SingleTrackDAO singleTrackDAO = factory.getSingleTrackDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<SingleTrack> singleTrackList = singleTrackDAO.loadAll();
        for (SingleTrack singleTrack : singleTrackList) {
            int nbTrains = 0;
            List<Fill> fillList = singleTrack.getFill();
            Collections.sort(fillList, new SortFillByDate());
            Datetime prev = new Datetime();
            for (Fill fill : fillList) {
                if (prev.getTime() != fill.getDate().getTime()) {
                    if (nbTrains > singleTrack.getCapacity())  {
                        logger.log(Level.INFO, "Single track capacity error : "
                                + singleTrack.getName() + " number of trains ("
                                + nbTrains + ") exceeds its capacity ("
                                + singleTrack.getCapacity() + ") at "
                                + prev.toString() + ".");
                        valid = false;
                    }
                }
                if (fill.getEvent().equals("EnterResource")) {
                    nbTrains++;
                } else if (fill.getEvent().equals("ExitResource")) {
                    nbTrains--;
                }
                prev = fill.getDate();
            }

            if (nbTrains > singleTrack.getCapacity())  {
                logger.log(Level.INFO, "Single track capacity error : "
                        + singleTrack.getName() + " number of trains ("
                        + nbTrains + ") exceeds its capacity ("
                        + singleTrack.getCapacity() + ") at "
                        + prev.toString() + ".");
                valid = false;
            }
        }
        return valid;
    }

}
