// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

public class UncoveredDeparturesCost implements Cost {

    private DAOFactory factory;

    public UncoveredDeparturesCost(DAOFactory factory) {
        this.factory = factory;
    }

    public float calculate() throws ROADEFException {
        HashMap<Departure, Train> assigned = new HashMap<Departure, Train>();
        float score = 0;
        TrainDAO dao = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals("Departure")) {
                    Departure dep = departureDAO.loadByName(fill.getComplement());
                    if (dep != null) {
                        assigned.put(dep, train);
                     }
                }
            }
        }

        for (Departure dep : departureDAO.loadAll()) {
            Train train = assigned.get(dep);
            if (train == null) {
                logger.log(Level.INFO, "Uncovered departures cost : "
                        + dep.getName() + " is uncovered.");
                score++;
            }
        }

        return score;
    }

}
