// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.bo;

import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;
import com.sncf.checker.io.PlatformDAO;

/**
 * Arrival.
 *
 * @author YDS
 *
 */
public class Arrival {

    /** dao for compatible platform list lazy loading. */
    private PlatformDAO platformDAO;

    /**
     * Constructor.
     */
    public Arrival() {
        prefPlat = new ArrayList<Platform>();
        sequenceList = new ArrayList<ArrDepSequence>();
    }

    /**
     * lazy-loading version constructor.
     *
     * @param occupationDAO
     * @param fillDAO
     */
    public Arrival(final PlatformDAO platformDAO) {
        this.platformDAO = platformDAO;
        sequenceList = new ArrayList<ArrDepSequence>();
    }

    /** name. */
    private String name;

    /**
     * Get the name.
     *
     * @return name
     */
    public final String getName() {
        return name;
    }

    /**
     * Set the name.
     *
     * @param name String to set
     */
    public final void setName(final String name) {
        this.name = name;
    }

    /** train. */
    private Train train;

    /**
     * Get the train.
     *
     * @return train
     */
    public final Train getTrain() {
        return train;
    }

    /**
     * Set the train.
     *
     * @param train to set
     */
    public final void setTrain(final Train train) {
        this.train = train;
        if (train != null) {
            if (train.getArrival() != this) {
                train.setArrival(this);
            }
        }
    }

    /** arrival time. */
    private Datetime arrTime;

    /**
     * Get the arrival time.
     *
     * @return arrival time
     */
    public final Datetime getArrTime() {
        return arrTime;
    }

    /**
     * Set the arrival time.
     *
     * @param arrTime arrival time
     */
    public final void setArrTime(final Datetime arrTime) {
        this.arrTime = arrTime;
    }

    /** arrival sequence. */
    private ArrayList<ArrDepSequence> sequenceList;

    /**
     * Get the arrival sequence.
     *
     * @return arrival sequence
     */
    public final List<ArrDepSequence> getArrSequence() {
        return sequenceList;
    }

    /**
     * Set the arrival sequence.
     *
     * @param sequence arrival sequence
     */
    public final void addArrSequence(final ArrDepSequence sequence) {
        sequenceList.add(sequence);
    }

    /** prefered platforms. */
    private ArrayList<Platform> prefPlat;

    /**
     * Get the prefered platforms.
     *
     * @return platform list
     */
    public final List<Platform> getPreferedPlatforms() {
        if (prefPlat == null) {
            prefPlat = new ArrayList<Platform>(platformDAO.loadCompatible(name));
        }
        return prefPlat;
    }

    /**
     * Add a prefered platform.
     *
     * @param platform to add
     */
    public final void addPreferedPlatform(final Platform platform) {
        if (prefPlat == null) {
            prefPlat = new ArrayList<Platform>(platformDAO.loadCompatible(name));
        }
        prefPlat.add(platform);
    }

    /** ideal dwell time. */
    private Duration idealDwell;

    /**
     * Get the ideal dwell time.
     *
     * @return ideal dwell time
     */
    public final Duration getIdealDwell() {
        return idealDwell;
    }

    /**
     * Set the ideal dwell time.
     *
     * @param duration ideal dwell time
     */
    public final void setIdealDwell(final Duration duration) {
        this.idealDwell = duration;
    }

    /** max dwell time. */
    private Duration maxDwell;

    /**
     * Get the max dwell time.
     *
     * @return remaining time
     */
    public final Duration getMaxDwell() {
        return maxDwell;
    }

    /**
     * Set the max dwell time.
     *
     * @param maxDwell max dwell time
     */
    public final void setMaxDwell(final Duration maxDwell) {
        this.maxDwell = maxDwell;
    }

    /** name. */
    private Departure linkedDep;

    /**
     * Get the name.
     *
     * @return name
     */
    public final Departure getLinkedDep() {
        return linkedDep;
    }

    /**
     * Set the name.
     *
     * @param name String to set
     */
    public final void setLinkedDep(final Departure linkedDep) {
        this.linkedDep = linkedDep;
        if (linkedDep != null) {
            if (linkedDep.getLinkedArr() != this) {
                linkedDep.setLinkedArr(this);
            }
        }
    }

    JointArrDep jointArrDep;

    public JointArrDep getJointArrDep() {
        return jointArrDep;
    }

    public void setJointArrDep(JointArrDep jointArrDep) {
        this.jointArrDep = jointArrDep;
    }
}
