// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.common;

import java.util.TreeMap;

/**
 * Basic implementation of multi keys map. Each value is associated to two keys.
 *
 * @author YDS
 *
 * @param <K1> first key type
 * @param <K2> second key type
 * @param <V> value type
 */
public final class TreeMultiKeyMap<K1, K2, V> {

    /** concrete container. */
    private TreeMap<K1, TreeMap<K2, V>> container;

    /**
     * Constructor.
     */
    public TreeMultiKeyMap() {
        container = new TreeMap<K1, TreeMap<K2, V>>();
    }

    /**
     * Clear the map.
     */
    public void clear() {
        container.clear();
    }

    /**
     * Return if the container is empty.
     *
     * @return true if empty, false otherwise
     */
    public boolean isEmpty() {
        return container.isEmpty();
    }

    /**
     * Insert a new value.
     *
     * @param key1 first key
     * @param key2 second key
     * @param value mapped value
     */
    public void put(final K1 key1, final K2 key2, final V value) {
        TreeMap<K2, V> map;

        map = container.get(key1);
        if (map == null) {
            map = new TreeMap<K2, V>();
            container.put(key1, map);
        }
        map.put(key2, value);
    }

    /**
     * Get a value.
     *
     * @param key1 first key
     * @param key2 second key
     * @return value
     */
    public V get(final K1 key1, final K2 key2) {
        V value = null;
        TreeMap<K2, V> map;

        map = container.get(key1);
        if (map != null) {
            value = map.get(key2);
        }

        return value;
    }

    /**
     * Remove a value.
     *
     * @param key1 first key
     * @param key2 second key
     */
    public void remove(final K1 key1, final K2 key2) {
        TreeMap<K2, V> map;

        map = container.get(key1);
        if (map != null) {
            map.remove(key2);
        }
    }

}
