// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.common;

import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

/**
 * Basic implementation of a multimap. A key can be associated with more than
 * a value.
 *
 * @author YDS
 *
 * @param <K> key type
 * @param <V> value type
 */
public final class TreeMultiMap<K, V> {

    /** concrete container. */
    private TreeMap<K, LinkedList<V>> container;

    /**
     * Constructor.
     */
    public TreeMultiMap() {
        container = new TreeMap<K, LinkedList<V>>();
    }

    /**
     * Clear the map.
     */
    public void clear() {
        container.clear();
    }

    /**
     * Return if the container is empty.
     *
     * @return true if empty, false otherwise
     */
    public boolean isEmpty() {
        return container.isEmpty();
    }

    /**
     * Insert a new value.
     *
     * @param key associated key
     * @param value mapped value
     */
    public void put(final K key, final V value) {
        LinkedList<V> list;

        list = container.get(key);
        if (list == null) {
            list = new LinkedList<V>();
            container.put(key, list);
        }
        list.add(value);
    }

    /**
     * Get associated values.
     *
     * @param key searched key
     * @return list of value, an empty list if no value is found
     */
    public List<V> get(final K key) {
        LinkedList<V> list;

        list = container.get(key);
        if (list == null) {
            list = new LinkedList<V>();
        }

        return list;
    }

    /**
     * Remove associated values.
     *
     * @param key searched key
     */
    public void remove(final K key) {
        container.remove(key);
    }
}
