// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

/**
 * Data Access Object Concrete factory for tabulation separated flat files.
 *
 * @author YDS
 *
 */
public class FlatFileDAOFactory implements DAOFactory {

    /** scenario file path. */
    private String scenario;
    /** solution file path. */
    private String solution;

    /**
     * Constructor
     *
     * @param scenario file path to the scenario zip archive.
     * @param solution file path to the solution file
     */
    public FlatFileDAOFactory(final String scenario, final String solution) {
        this.scenario = scenario;
        this.solution = solution;
    }

    private ArrDepSequenceFlatFileDAO sequenceDAO;

    /**
     * ArrDepSequenceDAO factory.
     *
     * @return ArrDepSequenceDAO
     */
    public ArrDepSequenceDAO getArrDepSequenceDAO() {
        if (sequenceDAO == null) {
            sequenceDAO = new ArrDepSequenceFlatFileDAO(scenario, solution);
            sequenceDAO.setTrackGroupDAO(getTrackGroupDAO());
        }
        return sequenceDAO;
    }

    private FillFlatFileDAO fillDAO;

    /**
     * FillDAO factory.
     *
     * @return FillDAO
     */
    public FillDAO getFillDAO() {
        if (fillDAO == null) {
            fillDAO = new FillFlatFileDAO(scenario, solution);
            fillDAO.setResourceDAO(getResourceDAO());
            fillDAO.setTrainDAO(getTrainDAO());
        }
        return fillDAO;
    }

    private MaintenanceFacilityFlatFileDAO maintenanceDAO;

    /**
     * MaintenanceFacilityDAO factory.
     *
     * @return MaintenanceFacilityDAO
     */
    public MaintenanceFacilityDAO getMaintenanceFacilityDAO() {
        if (maintenanceDAO == null) {
            maintenanceDAO = new MaintenanceFacilityFlatFileDAO(scenario, solution);
            maintenanceDAO.setGateDAO(getGateDAO());
            maintenanceDAO.setFillDAO(getFillDAO());
            maintenanceDAO.setTrainCategoryDAO(getTrainCategoryDAO());
        }
        return maintenanceDAO;
    }

    private PlatformFlatFileDAO platformDAO;

    /**
     * PlatformDAO factory.
     *
     * @return PlatformDAO
     */
    public PlatformDAO getPlatformDAO() {
        if (platformDAO == null) {
            platformDAO = new PlatformFlatFileDAO(scenario, solution);
            platformDAO.setGateDAO(getGateDAO());
            platformDAO.setFillDAO(getFillDAO());
            platformDAO.setTrainCategoryDAO(getTrainCategoryDAO());
        }
        return platformDAO;
    }

    private ResourceFlatFileDAO resourceDAO;

    /**
     * ResourceDAO factory.
     *
     * @return ResourceDAO
     */
    public ResourceDAO getResourceDAO() {
        if (resourceDAO == null) {
            resourceDAO = new ResourceFlatFileDAO(scenario, solution);
            resourceDAO.setYardSequenceDAO(getYardDAO());
            resourceDAO.setTrackGroupDAO(getTrackGroupDAO());
            resourceDAO.setIndividualTrackDAO(getIndividualTrackDAO());
        }
        return resourceDAO;
    }

    private SingleTrackFlatFileDAO singleTrackDAO;

    /**
     * SingleTrackDAO factory.
     *
     * @return SingleTrackDAO
     */
    public SingleTrackDAO getSingleTrackDAO() {
        if (singleTrackDAO == null) {
            singleTrackDAO = new SingleTrackFlatFileDAO(scenario, solution);
            singleTrackDAO.setGateDAO(getGateDAO());
            singleTrackDAO.setFillDAO(getFillDAO());
            singleTrackDAO.setTrainCategoryDAO(getTrainCategoryDAO());
        }
        return singleTrackDAO;
    }

    private TrackGroupFlatFileDAO trackGroupDAO;

    /**
     * TrackGroupDAO factory.
     *
     * @return TrackGroupDAO
     */
    public TrackGroupDAO getTrackGroupDAO() {
        if (trackGroupDAO == null) {
            trackGroupDAO = new TrackGroupFlatFileDAO(scenario, solution);
            trackGroupDAO.setGateDAO(getGateDAO());
            trackGroupDAO.setFillDAO(getFillDAO());
            trackGroupDAO.setTrainCategoryDAO(getTrainCategoryDAO());
        }
        return trackGroupDAO;
    }

    private TrainCategoryFlatFileDAO trainCategoryDAO;

    /**
     * TrainCategoryDAO factory.
     *
     * @return TrainCategoryDAO
     */
    public TrainCategoryDAO getTrainCategoryDAO() {
        if (trainCategoryDAO == null) {
            trainCategoryDAO = new TrainCategoryFlatFileDAO(scenario, solution);
        }
        return trainCategoryDAO;
    }

    private TrainFlatFileDAO trainDAO;

    /**
     * TrainDAO factory.
     *
     * @return TrainDAO
     */
    public TrainDAO getTrainDAO() {
        if (trainDAO == null) {
            trainDAO = new TrainFlatFileDAO(scenario, solution);
            trainDAO.setArrivalDAO(getArrivalDAO());
            trainDAO.setFillDAO(getFillDAO());
            trainDAO.setTrainCategoryDAO(getTrainCategoryDAO());
            trainDAO.setResourceDAO(getResourceDAO());
        }
        return trainDAO;
    }

    private GateFlatFileDAO gateDAO;

    /**
     * GateDAO factory.
     *
     * @return GateDAO
     */
    public GateDAO getGateDAO() {
        if (gateDAO == null) {
            gateDAO = new GateFlatFileDAO(scenario, solution);
            gateDAO.setResourceDAO(getResourceDAO());
        }
        return gateDAO;
    }

    private YardFlatFileDAO yardDAO;

    /**
     * YardDAO factory.
     *
     * @return YardDAO
     */
    public YardDAO getYardDAO() {
        if (yardDAO == null) {
            yardDAO = new YardFlatFileDAO(scenario, solution);
            yardDAO.setGateDAO(getGateDAO());
            yardDAO.setFillDAO(getFillDAO());
            yardDAO.setTrainCategoryDAO(getTrainCategoryDAO());
        }
        return yardDAO;
    }

    private ArrivalFlatFileDAO arrivalDAO;

    /**
     * ArrivalDAO factory.
     *
     * @return ArrivalDAO
     */
    public ArrivalDAO getArrivalDAO() {
        if (arrivalDAO == null) {
            arrivalDAO = new ArrivalFlatFileDAO(scenario, solution);
            arrivalDAO.setArrDepSequenceDAO(getArrDepSequenceDAO());
            arrivalDAO.setPlatformDAO(getPlatformDAO());
            arrivalDAO.setTrainDAO(getTrainDAO());
            arrivalDAO.setDepartureDAO(getDepartureDAO());
            arrivalDAO.setJointArrDepDAO(getJointArrDepDAO());
        }
        return arrivalDAO;
    }

    private DepartureFlatFileDAO departureDAO;

    /**
     * DepartureDAO factory.
     *
     * @return DepartureDAO
     */
    public DepartureDAO getDepartureDAO() {
        if (departureDAO == null) {
            departureDAO = new DepartureFlatFileDAO(scenario, solution);
            departureDAO.setArrDepSequenceDAO(getArrDepSequenceDAO());
            departureDAO.setPlatformDAO(getPlatformDAO());
            departureDAO.setTrainCategoryDAO(getTrainCategoryDAO());
            departureDAO.setJointArrDepDAO(getJointArrDepDAO());
        }
        return departureDAO;
    }

    private ParametersFlatFileDAO parametersDAO;

    public ParametersDAO getParametersDAO() {
        if (parametersDAO == null) {
            parametersDAO = new ParametersFlatFileDAO(scenario, solution);
        }
        return parametersDAO;
    }

    private IndividualTrackFlatFileDAO individualTrackDAO;

    public IndividualTrackDAO getIndividualTrackDAO() {
        if (individualTrackDAO == null) {
            individualTrackDAO = new IndividualTrackFlatFileDAO(scenario, solution);
            individualTrackDAO.setPlatformDAO(getPlatformDAO());
            individualTrackDAO.setSingleTrackDAO(getSingleTrackDAO());
            individualTrackDAO.setMaintenanceFacilityDAO(getMaintenanceFacilityDAO());
        }
        return individualTrackDAO;
    }

    private ImposedConsumptionFlatFileDAO imposedConsumptionDAO;

    public ImposedConsumptionDAO getImposedConsumptionDAO() {
        if (imposedConsumptionDAO == null) {
            imposedConsumptionDAO = new ImposedConsumptionFlatFileDAO(scenario, solution);
            imposedConsumptionDAO.setResourceDAO(getResourceDAO());
        }
        return imposedConsumptionDAO;
    }

    private ReuseFlatFileDAO reuseDAO;

    public ReuseDAO getReuseDAO() {
        if (reuseDAO == null) {
            reuseDAO = new ReuseFlatFileDAO(scenario, solution);
            reuseDAO.setArrivalDAO(getArrivalDAO());
            reuseDAO.setDepartureDAO(getDepartureDAO());
        }
        return reuseDAO;
    }

    private JointArrDepFlatFileDAO jointArrDepDAO;

    public JointArrDepDAO getJointArrDepDAO() {
        if (jointArrDepDAO == null) {
            jointArrDepDAO = new JointArrDepFlatFileDAO(scenario, solution);
        }
        return jointArrDepDAO;
    }
}
