// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.ImposedConsumption;
import com.sncf.checker.common.Datetime;

public class ImposedConsumptionFlatFileDAO extends FlatFileDAO implements ImposedConsumptionDAO {

    /** scenario file path. */
    private final String scenario;

    private List<ImposedConsumption> orderedCache = null;

    public ImposedConsumptionFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** Resource DAO. */
    private ResourceDAO resourceDAO;

    /**
     * Resource DAO setter.
     *
     * @param dao Resource DAO
     */
    public void setResourceDAO(final ResourceDAO dao) {
        this.resourceDAO = dao;
    }

    public List<ImposedConsumption> loadAll() {
        initialize();
        ArrayList<ImposedConsumption> list = new ArrayList<ImposedConsumption>();
        list.addAll(orderedCache);
        return list;
    }

    /**
     * Extract all trains from departures.csv in zip file.
     */
    private void initialize() {
        if (orderedCache == null) {
            orderedCache = new ArrayList<ImposedConsumption>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "imposedConsumptions.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";", -1);
                    ImposedConsumption temp;

                    temp = deserialize(splits);
                    orderedCache.add(temp);
                    temp.setResource(resourceDAO.loadByName(splits[0]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Train object.
     *
     * @param entry data extracted from the file
     * @return train
     */
    private ImposedConsumption deserialize(final String[] raw) {
        ImposedConsumption entity = new ImposedConsumption();

        if (!raw[1].equals("")) {
            entity.setBeg(new Datetime(raw[1]));
        }
        if (!raw[2].equals("")) {
            entity.setEnd(new Datetime(raw[2]));
        }
        if (!raw[3].equals("")) {
            entity.setNb(Integer.parseInt(raw[3]));
        }
        if (!raw[4].equals("")) {
            entity.setOriginGate(raw[4]);
        }
        if (!raw[5].equals("")) {
            entity.setDestinationGate(raw[5]);
        }
        if (!raw[6].equals("")) {
            entity.setEntranceTime(new Datetime(raw[6]));
        }
        return entity;
    }
}
