// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.JointArrDep;
import com.sncf.checker.bo.JointArrDepEntry;

public class JointArrDepFlatFileDAO extends FlatFileDAO implements JointArrDepDAO {

    /** scenario file path. */
    private final String scenario;
    private Map<String, JointArrDep> cache = null;

    public JointArrDepFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    public JointArrDep loadByName(String name) {
        initialize();
        JointArrDep entity = cache.get(name);
        return entity;
    }

    /**
     * Extract all trains from arrivals.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, JointArrDep>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "jointArrDep.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    JointArrDep temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Train object.
     *
     * @param entry data extracted from the file
     * @return train
     */
    private JointArrDep deserialize(final String[] raw) {
        JointArrDep entity = cache.get(raw[0]);
        if (entity == null) {
            entity = new JointArrDep();
            entity.setName(raw[0]);
        }

        JointArrDepEntry entry = new JointArrDepEntry();
        entry.setArrDep(raw[1]);
        entry.setPosition(Integer.parseInt(raw[2]));
        entity.addEntryList(entry);
        return entity;
    }
}
