// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.Parameters;
import com.sncf.checker.common.Duration;

public class ParametersFlatFileDAO extends FlatFileDAO implements ParametersDAO {

    /** scenario file path. */
    private final String scenario;
    Parameters cache = null;

    public ParametersFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    public Parameters getParameters() {
        initialize();
        return cache;
    }

    private void initialize() {
        if (cache == null) {

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "parameters.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                Parameters temp = new Parameters();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    deserialize(splits, temp);
                }
                cache = temp;
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    private void deserialize(final String[] entry, Parameters parameters) {

        if (entry[0].equals("junTime")) {
            parameters.setJunTime(new Duration(entry[1]));
        } else if (entry[0].equals("disjTime")) {
            parameters.setDisjTime(new Duration(entry[1]));
        } else if (entry[0].equals("revTime")) {
            parameters.setRevTime(new Duration(entry[1]));
        } else if (entry[0].equals("minAsbTime")) {
            parameters.setMinAsbTime(new Duration(entry[1]));
        } else if (entry[0].equals("junCost")) {
            parameters.setJunCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("disjCost")) {
            parameters.setDisjCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("remDCost")) {
            parameters.setRemDCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("remTCost")) {
            parameters.setRemTCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("platAsgCost")) {
            parameters.setPlatAsgCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("dwellCost")) {
            parameters.setDwellCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("reuseCost")) {
            parameters.setReuseCost(Float.parseFloat(entry[1]));
        } else if (entry[0].equals("minResTime")) {
            parameters.setMinResTime(new Duration(entry[1]));
        } else if (entry[0].equals("maxDwellTime")) {
            parameters.setMaxDwellTime(new Duration(entry[1]));
        } else if (entry[0].equals("maxMaint")) {
            parameters.setMaxMaint(Integer.parseInt(entry[1]));
        }
    }
}
